import java.util.Random;

/**
 * So this is the more 'elegant' version of NumberStatistics.
 * Not only that it's more elegant, but it's the right way to write code.
 * I split the implementation to 2 different classes, so you can see 
 * the difference in elegance, but also that you see the implications
 * on code maintenance.
 * For example, try changing the possible numbers or the number of tests
 * in both implementations, and see the difference for yourselves.
 *
 */
public class NumberStatisticsElegant {
	
	/**
	 * A 'final' variable is a variable that is initialized with a value,
	 * and that value will never change throughout the program.
	 * Whenever you write a program, go through your literals (numbers and strings).
	 * If a literal has a logical meaning, it might need to be a final variable (constant).
	 */
	/**
	 * A 'static' variable is a variable that is associated with the class, and not objects.
	 * A static variable exists (in memory) only once, no matter how many objects we created of that class.
	 * We will always have the 'final' variables static. Can you tell why?
	 */
	static final int NUMBER_OF_TESTS = 10000000;
	static final int NUMBER_OF_RESULTS = 10;
	
	public static void main(String[] args) {
		
		// initialize variables
		Random random = new Random();
		int counter = 0;
		int[] results = new int[NUMBER_OF_RESULTS]; // Things to remember:
													// 	1. An array is an object in Java.
													// 	2. When you see an assignment statement (a codeline with '='), first evaluate the 
													//		right side, then assign the result in the left side.
													//	3. The 'new' keyword creates ('instantiates') a new object in memory (in the 'heap').
													// 	4. After instantiating an object, the 'new' keyword takes the address of that object,
													//		and returns it to the left side (in this case: to 'results').
													//	5. 'results' is of type 'reference to an array of integers'. You can never access
													//		an object directly. Only through reference variables.
													//	6. After running this line, we will have an array object in the 'heap', and a reference
													// 		variable 'results' in the stack, pointing to that object.
													//	7. This variable has a scope: it can be accessed by any code written in this block (search
													//      up for the opening curly brace to know in which block you are), and after this line.
		
		do { // http://www.youtube.com/watch?v=Ra70O9nps6E
			
			// pick a random number
			int result = random.nextInt(NUMBER_OF_RESULTS);

			// summarize the result
			results[result]++;	// here we are using the reference variable to access the array object, and read the value of a specific
								//	cell in the array, and then increase its value by 1.			
			
		// repeat many times
		} while (++counter < NUMBER_OF_TESTS); // see that the ++ is to the left of 'counter', so that we first increase it by 1, and only
												// then check if it's still not too large.
												// it would be the same as writing 'counter++' a line above this one.

		// print the statistics <--- this comment is not necessary, because the method call explains itself
		printStatistics(results); // we are not passing the array as a parameter, but rather the array object's address (= reference).
									// passing a reference to an object as a parameter is called 'call by ref'. 
									// In some languages (C++ for example) you can pass the entire object as a parameter. 
									// In Java you can only pass primitive types and references to objects.
	}

	/**
	 * Since printing statistics of an array of numbers can be a separate logical algorithm, we prefer to write it as a different method.
	 * - 'private': because it is intended only for us (the class authors), and not other developers who use our class.
	 * - 'static': because this method does not change any instance variable. This class is a program anyway, it shouldn't have no-static methods.
	 * - 'void': if we don't want the method to return anything.
	 * - 'results': we ask for the results array because we are out of the scope of the original 'results' variable in the main method.
	 * @param results a reference to an array of integers - a parameter is like a local variable, that gets its value from the code that is calling our method. 
	 */
	private static void printStatistics(int[] results) {
		// print statistics about each number: summary / the number of tests
		for (int curr_result = 0; curr_result < NUMBER_OF_RESULTS; ++curr_result) {	// note, that since the right side ('++curr_result') has only one operation in it, it doesn't
																					// matter if we put the ++ to the left or to the right.
			System.out.println(curr_result + ": %" + 100d * results[curr_result] / NUMBER_OF_TESTS); // 
		}
	}
}
